-------------------------------------------------
-- LOVE: ParticleSystem demo.						
-- Website: http://love2d.org
-- Licence: ZLIB/libpng									
-- Copyright (c) 2006-2009 LOVE Development Team
-------------------------------------------------


systems = {}
current = 1

function love.load()
	part1 = love.graphics.newImage("part1.png");
	cloud = love.graphics.newImage("cloud.png");
	square = love.graphics.newImage("square.png")
	font = love.graphics.newFont(love._vera_ttf, 10)

	love.graphics.setFont(font)
	love.graphics.setColor(200, 200, 200);
	
	local p = love.graphics.newParticleSystem(part1, 1000)
	p:setEmissionRate(100)
	p:setSpeed(300, 400)
	p:setGravity(0)
	p:setSize(2, 1)
	p:setColor(255, 255, 255, 255, 58, 128, 255, 0)
	p:setPosition(400, 300)
	p:setLifetime(1)
	p:setParticleLife(1)
	p:setDirection(0)
	p:setSpread(360)
	p:setRadialAcceleration(-2000)
	p:setTangentialAcceleration(1000)
	p:stop()
	table.insert(systems, p)

	p = love.graphics.newParticleSystem(cloud, 1000)
	p:setEmissionRate(100)
	p:setSpeed(200, 250)
	p:setGravity(100, 200)
	p:setSize(1, 1)
	p:setColor(16, 81, 229, 255, 176, 16, 229, 0)
	p:setPosition(400, 300)
	p:setLifetime(1)
	p:setParticleLife(1)
	p:setDirection(180)
	p:setSpread(20)
	--p:setRadialAcceleration(-200, -300)
	p:stop()
	table.insert(systems, p)		

	p = love.graphics.newParticleSystem(square, 1000)
	p:setEmissionRate(60)
	p:setSpeed(200, 250)
	p:setGravity(100, 200)
	p:setSize(1, 2)
	p:setColor(240, 3, 176, 255, 204, 240, 3, 0)
	p:setPosition(400, 300)
	p:setLifetime(1)
	p:setParticleLife(2)
	p:setDirection(90)
	p:setSpread(0)
	p:setSpin(300, 800)
	p:stop()
	table.insert(systems, p)		

	p = love.graphics.newParticleSystem(part1, 1000)
	p:setEmissionRate(1000)
	p:setSpeed(300, 400)
	p:setSize(2, 1)
	p:setColor(220, 105, 20, 255, 194, 30, 18, 0)
	p:setPosition(400, 300)
	p:setLifetime(0.1)
	p:setParticleLife(0.2)
	p:setDirection(0)
	p:setSpread(360)
	p:setTangentialAcceleration(1000)
	p:setRadialAcceleration(-2000)
	p:stop()
	table.insert(systems, p)	

	p = love.graphics.newParticleSystem(part1, 1000)
	p:setEmissionRate(200)
	p:setSpeed(300, 400)
	p:setSize(1, 2)
	p:setColor(255, 255, 255, 255, 255, 128, 128, 0)
	p:setPosition(400, 300)
	p:setLifetime(1)
	p:setParticleLife(2)
	p:setDirection(0)
	p:setSpread(360)
	p:setTangentialAcceleration(2000)
	p:setRadialAcceleration(-8000)
	p:stop()
	table.insert(systems, p)
		
	fbo = love.graphics.newFbo(love.graphics.getWidth(), love.graphics.getHeight())

	local vs = love.graphics.newShader('vertex', love.filesystem.read("null.vertex.glsl"))
	local function makeProgram(fs_source)
		local program = love.graphics.newProgram()
		local fs = love.graphics.newShader('fragment', love.filesystem.read(fs_source))
		program:attachShader(vs, fs)
		program:link()
		return program
	end
	programs = {}
	programs.null   = makeProgram("null.fragment.glsl")
	programs.invert = makeProgram("invert.fragment.glsl")
	programs.color  = makeProgram("color.fragment.glsl")
	programs.sobel  = makeProgram("sobel.fragment.glsl")
	programs.blur   = makeProgram("blur.fragment.glsl")
	programs.glow   = makeProgram("glow.fragment.glsl")
	program = programs.null
	programs.name = {
		[programs.null]   = "null",
		[programs.invert] = "invert",
		[programs.color]  = "color",
		[programs.sobel]  = "sobel",
		[programs.blur]   = "blur",
		[programs.glow]   = "glow",
	}
end

direction = 0

function love.update(dt)

	if love.mouse.isDown("l") then
		systems[current]:setPosition(love.mouse.getX(), love.mouse.getY())
		systems[current]:start()
	end
	
	if love.keyboard.isDown("d") then
		systems[current]:setDirection(direction)
		direction = math.mod(direction + 90 * dt, 360)
	end
		
	systems[current]:update(dt)
end

function love.draw()
	fbo:render(function()
		love.graphics.setColorMode("modulate")
		love.graphics.setBlendMode("additive")
		
		love.graphics.draw(systems[current], 0, 0)
		love.graphics.print("System: [" .. current .. "/"..table.getn(systems).."] - " .. systems[current]:count() .. " particles. Shader: " .. programs.name[program], 30, 570);
		love.graphics.print("Click: spawn particles. Mousewheel: change system.", 30, 530);
		love.graphics.print("Press escape to exit. n,i,c,s,b and g change Shaders.", 30, 550);
	end)

	program:use()
	love.graphics.setColor(255,255,255)
	love.graphics.draw(fbo, 0, 0)
	program:unuse()

	if program == programs.invert then
		love.graphics.setColor(0,0,0)
	end
	love.graphics.print("System: [" .. current .. "/"..table.getn(systems).."] - " .. systems[current]:count() .. " particles. Shader: " .. programs.name[program], 30, 570);
	love.graphics.print("Click: spawn particles. Mousewheel: change system.", 30, 530);
	love.graphics.print("Press escape to exit. n,i,c,s,b and g change Shaders.", 30, 550);
end

function love.mousepressed(x, y, button)
	if button == "wu" then
		current = current + 1;
		if current > table.getn(systems) then current = table.getn(systems) end
	end

	if button == "wd" then
		current = current - 1;
		if current < 1 then current = 1 end
	end
end

function love.keypressed(key)
	if key == "escape" then
		love.event.push("q")
	end
	
	if key == "n" then
		program = programs.null
	elseif key == "i" then
		program = programs.invert
	elseif key == "c" then
		program = programs.color
	elseif key == "s" then
		program = programs.sobel
	elseif key == "b" then
		program = programs.blur
	elseif key == "g" then
		program = programs.glow
	end


	if key == "r" then
		love.filesystem.load("main.lua")()
		love.load()
	end
end
